%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Blackjack Markov Chain Toolbox, v1.0 (08 Jan 2009)
%%%
%%% This main file BJMCmainScript.m computes the player advantage for
%%%   - basic strategy
%%%   - Thorp's complete point count system
%%% using Markov chains as described in the manuscript 
%%%    "A Markov Chain Analysis of Blackjack Strategy"
%%%    by Michael B. Wakin and Christopher J. Rozell, 2004.
%%%    available at: http://inside.mines.edu/~mwakin/papers/mcbj.pdf
%%%
%%% Software Copyright 2009 by 
%%%    Michael B. Wakin, Colorado School of Mines, mwakin@mines.edu
%%%    Christopher J. Rozell, Georgia Tech, crozell@gatech.edu 
%%%
%%% This software is released under a Creative Commons license
%%% (Attribution 3.0 Unported): 
%%%     http://creativecommons.org/licenses/by/3.0/
%%% You are free:
%%%    * to Share  to copy, distribute and transmit the work
%%%    * to Remix  to adapt the work
%%% Under the following conditions:
%%%   * You must attribute the work by retaining this header information
%%%     crediting the original authors and linking to the original
%%%     manuscript.
%%%   * For any reuse or distribution, you must make clear to others the 
%%%     license terms of this work.
%%%   * Any of the above conditions can be waived if you get permission 
%%%     from the copyright holder.
%%%   * Nothing in this license impairs or restricts the author's 
%%%     moral rights.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%
%%% SETUP %%%
%%%%%%%%%%%%%

% Choose number of decks -- must be a multiple of 1/4
numDecks = 1;

% Set up global variables, etc.
setupVariables;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% COMPUTATIONS -- BASIC STRATEGY %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This is actually does not depend on the # of decks; just assumes
% uniform pdf for this hand.
strategy = setStrategy('basic');
deck = [1 1 1 1 1 1 1 1 1 4]/13; % Deck pdf, listed in order Ace through 10
[playerAdvantage] = advSingleHand(deck,strategy)
[playerInsAdvantage] = advInsurance(deck)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% COMPUTATIONS -- THORP STRATEGY %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute equilibrium probabilities and write to file
% This does not need to be re-done if it is already written to the file
[eq,HLI,low,med,high,rem] = equilib(numDecks,1);
% If already done, load eq0_25, or load eq0_5, or load eq1, or load eq2,...
numStates = length(HLI) - 1; % Last state is zeros

% For each state, compute the advantage for playing the hand
% and the advantage for taking insurance, and the level Thorp recommends
% to bet. This can be SLOW with multiple decks.
playerAdvs = zeros(1,numStates);
playerInsAdvs = zeros(1,numStates);
bets = zeros(1,numStates);
for ii = 1:numStates
  tic;
  deck = lmh2deckPdf(low(ii),med(ii),high(ii),numDecks);
  [strategy,bet] = setStrategy('thorp',HLI(ii));
  bets(ii) = bet;
  playerAdvs(ii) = advSingleHand(deck,strategy);
  playerInsAdvs(ii) = advInsurance(deck);
  if (mod(ii,10) == 0)
      fprintf('Done %d states out of %d.\n',ii,numStates);
      toc % elapsed time displayed is for one state (not 10)
  end
  if (mod(ii,1000) == 0)
    save simRunning 
  end  
end
HLI = HLI(1:numStates);
eq = eq(1:numStates);

%%%%%%%%%%%%%%%%
%%% ANALYSIS %%%
%%%%%%%%%%%%%%%%

% Compute overall long-term expected gain, assuming we take insurance 
% whenever HLI > 8, and that we use Thorp's recommended betting strategy.
totalExpectedGain = sum(bets.*playerAdvs.*eq) + sum(playerInsAdvs.*eq.*(HLI>8).*bets/2)

% Compute overall long-term expected gain, assuming we take insurance 
% whenever HLI > 8, and that we use unity bets on every hand. We see the 
% advantage typically disappears if the player cannot scale his bets.
totalExpectedGainUnityBets = sum(playerAdvs.*eq) + sum(playerInsAdvs.*eq.*(HLI>8).*1/2)

%%%%%%%%%%%%%
%%% PLOTS %%%
%%%%%%%%%%%%%

% HLI histogram
figure(1); clf;
fontSize = 16;
minHLI = -100;
maxHLI = 100;
samples = 101;
binSize = (maxHLI-minHLI)/(samples-1);
binCenters = minHLI:binSize:maxHLI;
binLeft = binCenters - binSize/2;
binRight = binCenters + binSize/2;
binTime = zeros(1,samples);
for bin = 1:samples
  bins = find((HLI > binLeft(bin)) & (HLI <= binRight(bin)));
  binTime(bin) = sum(eq(bins));  
end
bar(binCenters,binTime);
xlabel('HLI','FontSize',fontSize);
ylabel('\pi','FontSize',fontSize);
set(gca,'FontSize',fontSize);
axis([-100 100 0 0.25]);

% Scatter plot of HLI vs. player advantage (no insurance)
figure(2);clf;hold on;
plot(HLI,100*playerAdvs,'.','MarkerSize',1);
xlabel('HLI','FontSize',fontSize);
ylabel('Player advantage (%)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

% Consolidated plot of HLI vs. player advantage (with and w/o ins)
%          Use Thorp's range.
figure(3);clf;hold on;
minHLI = -100.001;
maxHLI = 100;
samples = 100;
binSize = (maxHLI-minHLI)/samples;
binLeft = minHLI:binSize:(maxHLI-binSize);
binRight = (minHLI+binSize):binSize:maxHLI;
binCenters = (binLeft+binRight)/2;
binAdv = zeros(1,samples);
noHLI = zeros(1,samples);
for bin = 1:samples
  bins = find((HLI > binLeft(bin)) & (HLI <= binRight(bin)));
  if (sum(eq(bins)) > 0)
    binAdv(bin) = sum(playerAdvs(bins).*eq(bins))/sum(eq(bins));  
  else
    noHLI(bin) = 1;
  end
end
hliVals = binCenters(find(1-noHLI));
advVals = binAdv(find(1-noHLI));
plot(hliVals,100*advVals);
binAdv = zeros(1,samples);
noHLI = zeros(1,samples);
iAdv = playerAdvs + 0.5.*(HLI>8).*playerInsAdvs;
for bin = 1:samples
  bins = find((HLI > binLeft(bin)) & (HLI <= binRight(bin)));
  if (sum(eq(bins)) > 0)
    binAdv(bin) = sum(iAdv(bins).*eq(bins))/sum(eq(bins));  
  else
    noHLI(bin) = 1;
  end
end
hliVals = binCenters(find(1-noHLI));
advVals = binAdv(find(1-noHLI));
plot(hliVals,100*advVals);
axis([-55 55 -7 17]);
axis off;
axis square;
%grid on;
%xlabel('HLI');
%ylabel('Player advantage (%)');
line([0 0],[-7 16],'Color','k')
line([-55 55],[0 0],'Color','k')
line([-50 -50],[-1/2 1/2],'Color','k');text(-54,1,'-50','FontSize',fontSize);
line([-25 -25],[-1/2 1/2],'Color','k');text(-29,1,'-25','FontSize',fontSize);
line([25 25],[-1/2 1/2],'Color','k');text(23,1,'25','FontSize',fontSize);
line([50 50],[-1/2 1/2],'Color','k');text(48,1,'50','FontSize',fontSize);
line([-3 3],[-5 -5],'Color','k');text(4,-5,'-5%','FontSize',fontSize);
line([-3 3],[5 5],'Color','k');text(4,5,'5%','FontSize',fontSize);
line([-3 3],[10 10],'Color','k');text(4,10,'10%','FontSize',fontSize);
line([-3 3],[15 15],'Color','k');text(4,15,'15%','FontSize',fontSize);
text(57,0,'HLI','FontSize',fontSize)
text(-12,16.5,'Advantage','FontSize',fontSize)
text(10,13.5,'with insurance','FontSize',fontSize);
text(27,6,'without insurance','FontSize',fontSize)

% Player advantage (%) vs. time spent (with insurance)
figure(4);clf;
minAdv = -0.1;
maxAdv = 0.35;
samples = 46;
binSize = (maxAdv-minAdv)/(samples-1);
binCenters = minAdv:binSize:maxAdv;
binLeft = binCenters - binSize/2;
binRight = binCenters + binSize/2;
iAdv = playerAdvs + 0.5.*(HLI>8).*playerInsAdvs;
binTime = zeros(1,samples);
for bin = 1:samples
  bins = find((iAdv > binLeft(bin)) & (iAdv <= binRight(bin)));
  binTime(bin) = sum(eq(bins));  
end
bar(100*binCenters,binTime);
xlabel('Player advantage (%)','FontSize',fontSize);
ylabel('Relative time spent','FontSize',fontSize);
set(gca,'FontSize',fontSize);
axis([-11 36 0 0.35]);

% Expected gain vs. time spent
figure(5);clf;
G = bets.*playerAdvs + playerInsAdvs.*(HLI>8).*bets/2;
minG = -0.1;
maxG = 0.35;
samples = 46;
binSize = (maxG-minG)/(samples-1);
binCenters = minG:binSize:maxG;
binLeft = binCenters - binSize/2;
binRight = binCenters + binSize/2;
binTime = zeros(1,samples);
for bin = 1:samples
  bins = find((G > binLeft(bin)) & (G <= binRight(bin)));
  binTime(bin) = sum(eq(bins));  
end
bar(binCenters,binTime);
xlabel('Expected gain (dollars)','FontSize',fontSize);
ylabel('Relative time spent','FontSize',fontSize);
set(gca,'FontSize',fontSize);
axis([-0.11 0.36 0 0.35]);


